#!/bin/sh

#
# scan-pci.sh - affiche le type de carte detectee
#
# Usage : scan-pci.sh <fic liste pci> [-v]
#
# Description :
# scan-pci.sh affiche le type de carte detectee, et le module a utiliser.
# Le type de carte cherche est obtenu en cherchant dans un fichier donne
# en argument. Par exemple :
# 	niclist : affiche le module de la carte Ethernet
# 	vidlist : affiche le serveur X a utiliser
# 	sndlist : affiche le module de la carte son
#
# Options :
# <fic liste pci> : base de donnee associant a un ID PCI un modele de
# carte
# -v affiche plus d'infos
#
# Requiere :
# pnpdump(8)

[ $# -ne 1 -a $# -ne 2 ] && echo >&2 "Usage : $0 <fic liste pci> [-v]
Par exemple :

scan-pci.sh /etc/niclist    : affiche le module de la carte Ethernet
scan-pci.sh /etc/pcmcialist : affiche le module de la carte Ethernet PCMCIA
scan-pci.sh /etc/vidlist    : affiche le serveur X a utiliser
scan-pci.sh /etc/sndlist    : affiche le module de la carte son
" && exit 1

cd `dirname $0`

[ ! -r $1 -a -r /etc/$1 ] && Fic=/etc/$1 || Fic=$1
[ ! -r $Fic ] && echo >&2 "Erreur : $Fic n'est pas un fichier lisible" && exit 1

case "$2" in
    -v) Verbose="v1" ;;
    --v-simple) Verbose="v2" ;;
    "") ;;
    *) echo >&2 "Erreur : option non reconnue ($2)" && exit 1 ;;
esac

rm -rf /tmp/scan-pci
rm -rf /tmp/scan-pci.found

Pwd=`dirname $0`

if [ -r /usr/src/linux/drivers/pci/pci.ids ]
then
    Ids=/usr/src/linux/drivers/pci/pci.ids
elif [ -r /usr/share/pci.ids ]
then
    Ids=/usr/share/pci.ids
elif [ -r /etc/pci.ids ]
then
    Ids=/etc/pci.ids
else
    Ids=$Pwd/pci.ids
fi

if [ -r /etc/isalist ]
then
    IsaList=/etc/isalist
elif [ -r $Pwd/isalist ]
then
    IsaList=$Pwd/isalist
fi

NombreLignes=`cat $Fic | wc -l`
NoLigne=0

AfficheDesc ()
{
    Part1=`echo $1 | cut -d: -f1`
    Part2=`echo $1 | cut -d: -f2`
    Desc1=`grep "^$Part1" $Ids | sed "s/$Part1 *//"`
    Desc2=`grep "^[ 	]*$Part2 *[a-zA-Z]" $Ids | sed "s/$Part2[ ]*//"`
    if [ -n "$Desc1" ]
    then
        echo -en "\033[1;32mDescription : \033[0m"
        echo -e "\033[1;34m $Desc1 $Desc2\033[0m"
        return 0
    else
        return 1
#        shift 2
#        echo $*
    fi
}

cat $Fic | while read ligne
do
    NoLigne=`expr $NoLigne + 1`
    if [ "$Verbose" = "v1" ]
    then
    echo -en "\r\t\t\t\t\t\t\t\t`expr $NoLigne \* 100 / $NombreLignes` % complete\r"
    elif [ "$Verbose" = "v2" ]
    then
        echo "`expr $NoLigne \* 100 / $NombreLignes`"
    fi

    [ -z "$ligne" ] && continue
    set -- $ligne
    Id=`echo $1 | tr -d ":"`
    Mod="$2"
    if cat /proc/bus/pci/devices | grep -i "$Id" > /dev/null
    then
        [ -n "$Verbose" ] && echo
#        echo "$Mod" > /tmp/scan-pci
        AfficheDesc "$@" && echo "$Mod" > /tmp/scan-pci
        touch /tmp/scan-pci.found
#        touch /tmp/scan-pci.found
        break
    fi
done

#
# On n'a pas trouve de PCI, may be ISA ?
#

if [ ! -e /tmp/scan-pci.found ] && which pnpdump > /dev/null 2>&1
then
    [ `id -u` -ne 0 ] && echo -e "\nIl faut le mot de passe root pour executer pnpdump" && su root -c "pnpdump > /tmp/pnpdump.out" || pnpdump > /tmp/pnpdump.out

    NombreLignes=`cat $IsaList | wc -l`
    NoLigne=0

    if [ "$Verbose" = "v1" ]
    then
        echo -en "\nScanning PCI failed. Trying ISA..."
    fi
    cat $IsaList | while read ligne
    do
        NoLigne=`expr $NoLigne + 1`
        if [ "$Verbose" = "v1" ]
        then
            echo -en "\r\t\t\t\t\t\t\t\t`expr $NoLigne \* 100 / $NombreLignes` % complete\r"
        elif [ "$Verbose" = "v2" ]
        then
            echo "`expr $NoLigne \* 100 / $NombreLignes`"
        fi

        echo "$ligne" | grep "^\/" > /dev/null && Driver="$ligne"
        if echo "$ligne" | grep "^ISAPNP: " > /dev/null
        then
            Isa=`echo "$ligne" | sed "s/ISAPNP: \(.*\)=.*/\1/"`
            if grep "^# Vendor Id $Isa" /tmp/pnpdump.out > /dev/null
            then
                Driver=`basename $Driver`
                echo "$Driver" >> /tmp/scan-pci
                [ -n "$Verbose" ] && echo
                echo -en "\033[1;32mDescription :  "
#                grep "# *ANSI string " /tmp/pnpdump.out | sed "s/# *ANSI string //"
                egrep "^# Vendor Id $Isa|# *ANSI string " /tmp/pnpdump.out | \
                while read ligne
                do
                    echo "$ligne" | grep "$Isa" > /dev/null && read ligne2 && echo -en "\033[1;34m$ligne2" | sed "s/# *ANSI string //"
                done
                echo -en "\033[0m"
                break
            fi
        fi
    done
    rm -f /tmp/pnpdump.out
fi

NotFound=1
if [ -f /tmp/scan-pci ]
then
    echo -en "\033[1;32mModule      :  \033[0m"
    echo -en "\033[1;34m"
    set -- `cat /tmp/scan-pci`
    echo $*
    echo -en "\033[0m"
    rm -rf /tmp/scan-pci.found
    NotFound=0
elif [ "`basename $Fic`" = "niclist" ]
then
    echo >&2 "No PCI card found. Looking for an Ethernet controller..."
    echo -en "\033[1;34m"
    cat /proc/pci | grep "Ethernet controller: " && NotFound=0
    [ $NotFound -eq 0 ] && cat /proc/isapnp | grep -i "ether" && NotFound=0
    echo -en "\033[0m"
elif [ "`basename $Fic`" = "vidlist" ]
then
    echo -en "\033[1;34m"
    cat /proc/pci | grep "VGA compatible controller: " && NotFound=0
    echo -en "\033[0m"
fi

#    [ -n "$Verbose" ] && echo
if [ $NotFound -eq 1 ]
then
    [ -n "$Verbose" ] && echo
    echo -e "\033[1;31mNo card found\033[0m"
    exit 1
fi

grep `cat /tmp/scan-pci` $Fic > /dev/null
